/* --COPYRIGHT--,BSD
 * Copyright (c) 2021, Texas Instruments Incorporated
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * *  Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * *  Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * *  Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * --/COPYRIGHT--*/
//*****************************************************************************
//         MCT8316ZTEVM GUI Code Demo
//
// Texas Instruments, Inc.
// Aaron Barrera
// ******************************************************************************

#include <msp430.h>
#include <stdbool.h>
#include <stdint.h>
#include <HAL.h>
#include <GUI.h>
#include <math.h>
#include <Include/GUIComm.h>
#include <Include/QmathLib.h>
#include <Include/callbacks.h>
#include <Include/mct8316z_spi.h>

//This code controls a PWM duty cycle from the ADC value from the pot, and includes FGOUT frequency and Motor Speed readback.
//P1.1 = ADC (from pot)
//P2.0 = TB1.1 (PWM based on pot)
//P2.1 = TB1.2 (motor speed based from FGOUT)

void SPI_Init(void);
void GPIO_Init(void);
void Timer_B_Init(void);
void ADC_Init(void);
unsigned int getFGvalue(unsigned int fg0, unsigned int fg1);

// Q8 is used in this demo because the variable qCounter ranges 0.0-100.0
#define GLOBAL_Q 8

#define COMM_FREQ                    20000
#define NUM_POLES                    4
#define BUFFER_SIZE                  5
#define UPDATE_COUNT                 100

unsigned int ADC_Result;
unsigned int count = 0, risingEdge = 0, fallingEdge = 0;
float FGOUT_DC = 0.0;
float PWM_DC = 50.0, old_PWM_DC = 50.0;
unsigned int FGOUT_freq[5], commSpeed, FGOUT_sum = 0, FGOUT_avg;
unsigned int buffer_index = 0, update_count = 0;

// Global variable use to track the of state of GUI and counter values
volatile bool fgout_flag = false;
volatile bool adc_flag = false;
volatile bool flag = false;

long rise, fall, temp, diff, prev_rise;
unsigned int overflowCount = 0, riseCount = 0;

volatile bool nFAULT_flag = 0; nFAULT_updated = 0;

unsigned char RXData =0;
unsigned char TXData = 0;
unsigned char drvAddr = 0x03;
unsigned char drvData = 0x03;

int FGOUT_DIV = 1;

//! \brief RX Command structure.
//!         The corresponding callback will be called when the command is
//!         received from GUI.
//! Note: shorter command names take less time to process
const tGUI_RxCmd GUI_RXCommands[] =
{
     {"nSLEEP",    GUICallback_nSLEEP},
     {"nFAULT",    GUICallback_nFAULT},
     {"pwm_sel",    GUICallback_PwmSel},
     {"pwm_dc",    GUICallback_PwmDuty},
     {"num_pol",    GUICallback_NumPol},
     {"addr",    GUICallback_Addr},
     {"data",    GUICallback_Data},

     //SPI Commands
     {"readFlag",    GUICallback_Read},
     {"writeFlag",    GUICallback_Write},
     {"writeAllFlag",    GUICallback_WriteAll},
     {"readAllFlag",    GUICallback_ReadAll},

     //CTRL1
     {"REG_LOCK2", GUICallback_Reg_Lock2},
     {"REG_LOCK1", GUICallback_Reg_Lock1},
     {"REG_LOCK0", GUICallback_Reg_Lock0},

     //CTRL2
     {"SLEW1", GUICallback_Slew1},
     {"SLEW0", GUICallback_Slew0},
     {"PWM_MODE1", GUICallback_Pwm_Mode1},
     {"PWM_MODE0", GUICallback_Pwm_Mode0},
     {"CLR_FLT", GUICallback_Clr_Flt},

     //CTRL3
     {"OVP_SEL", GUICallback_Ovp_Sel},
     {"OVP_EN", GUICallback_Ovp_En},
     {"SPI_FLT_REP", GUICallback_Spi_Flt_Rep},
     {"OTW_REP", GUICallback_Otw_Rep},

     //CTRL4
     {"OCP_CBC", GUICallback_Ocp_Cbc},
     {"OCP_DEG1", GUICallback_Ocp_Deg1},
     {"OCP_DEG0", GUICallback_Ocp_Deg0},
     {"OCP_RETRY", GUICallback_Ocp_Retry},
     {"OCP_LVL", GUICallback_Ocp_Lvl},
     {"OCP_MODE1", GUICallback_Ocp_Mode1},
     {"OCP_MODE0", GUICallback_Ocp_Mode0},

     //CTRL5
     {"BEMF_TH", GUICallback_Bemf_Th},
     {"ILIM_RECIR", GUICallback_Ilim_Recir},
     {"AD_COMP_TH_LS", GUICallback_Ad_Comp_Th_Ls},
     {"AD_COMP_TH_HS", GUICallback_Ad_Comp_Th_Hs},
     {"EN_AAR", GUICallback_En_Aar},
     {"EN_ASR", GUICallback_En_Asr},
     {"CSA_GAIN1", GUICallback_Csa_Gain1},
     {"CSA_GAIN0", GUICallback_Csa_Gain0},

     //CTRL6
     {"BUCK_SR", GUICallback_Buck_SR},
     {"BUCK_PS_DIS", GUICallback_Buck_Ps_Dis},
     {"BUCK_SEL1", GUICallback_Buck_Sel1},
     {"BUCK_SEL0", GUICallback_Buck_Sel0},
     {"BUCK_DIS", GUICallback_Buck_Dis},

     //CTRL7
     {"HALL_HYS", GUICallback_Hall_Hys},
     {"BRAKE_MODE", GUICallback_Brake_Mode},
     {"COAST", GUICallback_Coast},
     {"BRAKE", GUICallback_Brake},
     {"DIR", GUICallback_Dir},

     //CTRL8
     {"FG_MODE1", GUICallback_Fg_Mode1},
     {"FG_MODE0", GUICallback_Fg_Mode0},
     {"DIS_MTR_LOCK", GUICallback_Dis_Mtr_Lock},
     {"MOTOR_LOCK_RETRY", GUICallback_Motor_Lock_Retry},
     {"MOTOR_LOCK_TDET1", GUICallback_Motor_Lock_Tdet1},
     {"MOTOR_LOCK_TDET0", GUICallback_Motor_Lock_Tdet0},
     {"MOTOR_LOCK_MODE1", GUICallback_Motor_Lock_Mode1},
     {"MOTOR_LOCK_MODE0", GUICallback_Motor_Lock_Mode0},

     //CTRL9
     {"ADVANCE2", GUICallback_Advance2},
     {"ADVANCE1", GUICallback_Advance1},
     {"ADVANCE0", GUICallback_Advance0},

};

//! \brief Increments counters using MSP430 and sends data to GUI.
//!
//! \return none
void main(void)
{
    WDTCTL = WDTPW | WDTHOLD;   // stop watchdog timer

    // Initialize clock, GPIOs
    HAL_System_Init();

    // Initialize GUI layer
    GUI_Init();
    GUI_InitRxCmd( &GUI_RXCommands[0],
                  (sizeof(GUI_RXCommands)/sizeof(GUI_RXCommands[0])) );

    SPI_Init();
    GPIO_Init();

    PM5CTL0 &= ~LOCKLPM5;

    ADC_Init();
    Timer_B_Init();

    SPI_SetupDrv();

    //Preset Initializations
    PwmSel = 1;
    ReadAllFlag = 0;
    GUIComm_sendUInt8("num_pol", 7, 4);
    GUIComm_sendBool("LED1", 4, 1);
    GUIComm_sendBool("LED2", 4, 0);
    GUIComm_sendBool("nFAULT", 6, 0);


    __bis_SR_register(LPM0_bits | GIE); // Enter LPM0 w/interrupt
    __enable_interrupt();

    while (1)
    {
        count++;
        ADCCTL0 |= ADCENC | ADCSC;            // Sampling and conversion start

        //Check for ADC-PWM flag
        if (adc_flag == true){
            adc_flag = false;
            if (PwmSel == 1)
            {
                TB1CCR1=(int)(ADC_Result/10);
            }
            else
            {
                TB1CCR1=(int)(PwmDuty*4.0);
            }
            old_PWM_DC = PWM_DC;
            //PWM_DC = (float)(((float)(TB1CCR1)*100.0)/(float)(TB1CCR0));
            PWM_DC = (float)((float)(ADC_Result)*1.0175/40.96);
            if (PWM_DC > 100.0)
            {
                PWM_DC = 100.0;
            }

            if (PWM_DC != old_PWM_DC)
            {
                GUIComm_sendInt16("pwm_dc", 6, (_q5)(PWM_DC*32.0));
            }
        }

        //Check for FGOUT / Motor Speed flag
        if (fgout_flag == true){
            fgout_flag = false;
            FGOUT_DIV = getFGvalue(FG_MODE0,FG_MODE1);
            //FGOUT_freq[buffer_index++] = (float)(32500.0*(float)(FGOUT_DIV)/(float)(diff));
            FGOUT_freq[buffer_index++] = (32500*FGOUT_DIV)/diff;

            if (buffer_index == BUFFER_SIZE)
            {
                buffer_index = 0;
                FGOUT_sum = 0;

                int i;

                for(i=0;i<BUFFER_SIZE;i++)
                {
                     FGOUT_sum += FGOUT_freq[i];
                }

                FGOUT_avg = FGOUT_sum/BUFFER_SIZE;
                //commSpeed = (float)((121.0*FGOUT_avg)/((float)NumPoles));
                commSpeed = (121.0*FGOUT_avg)/NumPoles;
            }

            update_count++;
            if (update_count == UPDATE_COUNT)
            {
                update_count = 0;
                GUIComm_sendUInt16("fgout", 5, (uint16_t)(FGOUT_avg));
                GUIComm_sendUInt16("rpm", 3, (uint16_t)(commSpeed));
            }
        }

        //Check for nSLEEP flag
        //nSLEEP ON
        if (nSLEEP_on_flag == true && nSLEEP_off_flag == false)
        {
            P3OUT &= ~BIT2;
        }

        //nSLEEP OFF
        if (nSLEEP_on_flag == false && nSLEEP_off_flag == true)
        {
            P3OUT |= BIT2;
        }

        //Check for nFAULT flag
        if (nFAULT_updated == false)
        {
            nFAULT_updated = true;
            if (nFAULT_flag == true)
            {
                GUIComm_sendBool("nFAULT", 6, 1);
            }
            else
            {
                GUIComm_sendBool("nFAULT", 6, 0);
            }
        }


        //Read SPI
        if (ReadFlag == 1)
        {
            ReadFlag = 0;

            unsigned short receivedData;
            receivedData = SPI_Read(UartAddr);

            UpdateSpiReg(UartAddr,receivedData);
        }

        //Read All Registers
        if (ReadAllFlag == 1)
        {
            ReadAllFlag = 0;
            SPI_Read_All();
        }

        __bis_SR_register(LPM0_bits); // Enter LPM0 w/interrupt
        __no_operation();

    }
}

void GPIO_Init(void)
{
    // Initialize variables
    P2DIR &= ~BIT1;                     // P2.1 input for FGOUT
    P2REN |= BIT1;                      // Enable pullup
    P2OUT |= BIT1;
    P2IES &= ~BIT1;                     // P2.1 rising edge
    P2IFG &= ~BIT1;
    P2IE  |= BIT1;                      // Enable interrupt

    P2DIR |= BIT0 ;                     // P2.0 output for PWM
    P2SEL0 |= BIT0;                     // Set as TB1.CCI2A pin (0b01), second function
    //P2SEL1 &= ~BIT1;                  // Set as TB1.CCI2A pin, second function

    P1SEL0 |= BIT1;                     // Configure Potentiometer as ADC A1 pin
    P1SEL1 |= BIT1;

    P1DIR |= BIT2;                      // Set P1.2 as output (LED1)
    P1OUT |= BIT2;                      // P1.2 high

    P3DIR |= BIT2;                      //nSLEEP as output
    P3OUT |= BIT2;                      //make nSLEEP high

    P3DIR &= ~BIT3;                      //nFAULT as input with pullup
    P3REN |= BIT3;
    P3OUT |= BIT3;
    P3IES |= BIT3;
    P3IE |= BIT3;
}

void ADC_Init(void)
{
// Configure ADC12
   ADCCTL0 |= ADCSHT_2 | ADCON;                             // ADCON, S&H=16 ADC clks
   ADCCTL1 |= ADCSHP;                                       // ADCCLK = MODOSC; sampling timer
   ADCCTL2 &= ~ADCRES;                                      // clear ADCRES in ADCCTL
   ADCCTL2 |= ADCRES_2;                                     // 12-bit conversion results
   ADCMCTL0 |= ADCINCH_1;                                   // A1 ADC input select; Vref=AVCC
   ADCIE |= ADCIE0;                                         // Enable ADC conv complete interrupt
}

void Timer_B_Init(void)
{
    TB1CCR0 = 400;                         // PWM Period (8MHz / 400 = 20kHz)
    TB1CCTL1 = OUTMOD_7 | CLLD_1;          // CCR1 reset/set
    TB1CCR1 = 200;                         // CCR1 PWM duty cycle
    TB1CTL = TBSSEL_2 | MC_1 | TBCLR;      // SMCLK, up mode, clear TBR

    //Timer B0 oconfiguration - to use in continuous mode and count when FGOUT changes
    TB0CTL = TBSSEL_1 | MC_2 | TBIE;                     // ACLK, cont mode
}

void SPI_Init(void)
{
    P4SEL0 |= BIT5 | BIT6 | BIT7;             // set 3-SPI pin as second function

    UCB1CTLW0 |= UCSWRST;                     // **Put state machine in reset**
    UCB1CTLW0 |= UCMST|UCSYNC|UCMSB;   // 3-pin, 8-bit SPI master
                                              // Clock polarity high, MSB
    UCB1CTLW0 |= UCSSEL__ACLK;                // Select ACLK
    UCB1BR0 = 0x02;                           // BRCLK = ACLK/2
    UCB1BR1 = 0;                              //
    //UCB1MCTLW = 0;                            // No modulation
    UCB1CTLW0 &= ~UCSWRST;                    // **Initialize USCI state machine**
    UCB1IE |= UCRXIE;                         // Enable USCI_B1 RX interrupt
    //TXData = 0x01;                            // Holds TX data

    PM5CTL0 &= ~LOCKLPM5;                     // Disable the GPIO power-on default high-impedance mode
                                              // to activate previously configured port settings
}

unsigned int getFGvalue(unsigned int fg0, unsigned int fg1)
{
    unsigned int div;
    if (!fg0 && !fg1)
    {
        div = 1;
    }
    else if (fg0 && !fg1)
    {
        div = 2;
    }
    else if (!fg0 && fg1)
    {
        div = 4;
    }
    else if (fg0 && fg1)
    {
        div = 8;
    }

    return (div);
}

//**************************************INTERRUPTS***********************************************

// ADC interrupt service routine
#if defined(__TI_COMPILER_VERSION__) || defined(__IAR_SYSTEMS_ICC__)
#pragma vector=ADC_VECTOR
__interrupt void ADC_ISR(void)
#elif defined(__GNUC__)
void __attribute__ ((interrupt(ADC_VECTOR))) ADC_ISR (void)
#else
#error Compiler not supported!
#endif
{
    switch(__even_in_range(ADCIV,ADCIV_ADCIFG))
    {
        case ADCIV_NONE:
            break;
        case ADCIV_ADCOVIFG:
            break;
        case ADCIV_ADCTOVIFG:
            break;
        case ADCIV_ADCHIIFG:
            break;
        case ADCIV_ADCLOIFG:
            break;
        case ADCIV_ADCINIFG:
            break;
        case ADCIV_ADCIFG:
            adc_flag = 1;
            ADC_Result = ADCMEM0;
            __bic_SR_register_on_exit(LPM0_bits);            // Clear CPUOFF bit from LPM0
            break;
        default:
            break;
    }
}

#if defined(__TI_COMPILER_VERSION__) || defined(__IAR_SYSTEMS_ICC__)
#pragma vector=USCI_B1_VECTOR
__interrupt void USCI_B1_ISR(void)
#elif defined(__GNUC__)
void __attribute__ ((interrupt(USCI_A0_VECTOR))) USCI_A0_ISR (void)
#else
#error Compiler not supported!
#endif
{
  switch(__even_in_range(UCB1IV,USCI_SPI_UCTXIFG))
  {
    case USCI_NONE: break;                    // Vector 0 - no interrupt
    case USCI_SPI_UCRXIFG:
           RXData = UCB1RXBUF;
           UCB1IFG &= ~UCRXIFG;
           __bic_SR_register_on_exit(LPM0_bits);// Wake up to setup next TX
           break;
    case USCI_SPI_UCTXIFG:
          UCB1TXBUF = TXData;                // Transmit characters
          UCB1IE &= ~UCTXIE;
          break;
    default: break;
  }
}

// Timer0_B3 Interrupt Vector (TBIV) handler
#if defined(__TI_COMPILER_VERSION__) || defined(__IAR_SYSTEMS_ICC__)
#pragma vector=TIMER0_B1_VECTOR
__interrupt void TIMER0_B1_ISR(void)
#elif defined(__GNUC__)
void __attribute__ ((interrupt(TIMER0_B1_VECTOR))) TIMER0_B1_ISR (void)
#else
#error Compiler not supported!
#endif
{
  switch(__even_in_range(TB0IV,TB0IV_TBIFG))
  {
      case TB0IV_NONE:
          break;                             // No interrupt
      case TB0IV_TBCCR1:
          break;                             // CCR1 not used
      case TB0IV_TBCCR2:
          break;                             // CCR2 not used
      case TB0IV_TBIFG:                      // overflow
          overflowCount++;
          __bic_SR_register_on_exit(LPM0_bits);// Exit LPM4 on reti
          break;
      default:
          break;
  }
}

// Timer1_B1 CCR2, TB Interrupt Handler
#if defined(__TI_COMPILER_VERSION__) || defined(__IAR_SYSTEMS_ICC__)
#pragma vector = TIMER1_B1_VECTOR
__interrupt void TIMER1_B1_ISR(void)
#elif defined(__GNUC__)
void __attribute__ ((interrupt(TIMER1_B1_VECTOR))) TIMER1_B1_ISR (void)
#else
#error Compiler not supported!
#endif

{
  switch(__even_in_range(TB1IV,TB1IV_TBIFG))
  {
      case TB1IV_NONE:
          break;                             // No interrupt
      case TB1IV_TBCCR1:
          TB1CCTL1 &= ~CCIFG;
          __bic_SR_register_on_exit(LPM0_bits);            // Clear CPUOFF bit from LPM0
          break;                             // CCR1 not used
      case TB1IV_TBCCR2:
          TB1CCTL2 &= ~CCIFG;
          P1OUT ^= BIT2;
          flag ^= 1;
          if (!flag)
          {

              risingEdge = TB1CCR2;

          }
          else
          {
              fallingEdge = TB1CCR2;
          }
          __bic_SR_register_on_exit(LPM0_bits);// CPU active on reti
          break;                             // CCR2 not used
      case TB1IV_TBIFG:                      // overflow
          break;
      default:
          break;
  }
}

#pragma vector=PORT2_VECTOR
__interrupt void Port2_ISR(void){
    if(P2IFG & BIT1)
    {
        prev_rise = rise;
        rise = (65535*overflowCount) + TB0R;
        diff = rise - prev_rise;
        overflowCount = 0;
        P1OUT ^= BIT2;
        flag ^= 1;
        if (flag == true)
        {
            fgout_flag = 1;
            TB1CTL |= TBCLR; // Re-start the timer
        }
    }
    P2IFG &= ~BIT1;
    __bic_SR_register_on_exit(LPM0_bits);// Exit LPM4 on reti
}

#pragma vector=PORT3_VECTOR
__interrupt void Port3_ISR(void){
    if(P3IFG & BIT3)
    {
        nFAULT_flag ^= 1;      //toggle nFAULT flag
        nFAULT_updated = false;          //need to update nFAULT LED on GUI
        if (nFAULT_flag == true)
        {
            P3IES &= ~BIT3;              //change to rising edge
        }
        else
        {
            P3IES |= BIT3;              //change to falling edge
        }
    }
    P3IFG &= ~BIT3;
    __bic_SR_register_on_exit(LPM0_bits);// Exit LPM4 on reti
}

